from setuptools import setup, find_packages

setup(
   name='ss_proxy_pool',
   version='3.1',
   packages=find_packages(),
   author='Example Author',
   author_email='',
   keywords='',
   description='A small example package',
   license='',
   url='',
   include_package_data=True,
   install_requires=[],
)
